@@ -10,9 +10,10 @@
 #' attributes common to those graphs.
 #'
 #' In addition to creating the initial \code{igraph} graphs from the
-#' connectivity matrices, \code{\link{set_brainGraph_attr}} will be called on
-#' each graph if \code{set.attrs=TRUE}; other arguments will be passed to that
-#' function. You may display a progress bar by setting \code{.progress=TRUE}.
+#' connectivity matrices, then attributes will be calculated and assigned for
+#' each graph via \code{\link{set_brainGraph_attr}} if \code{set.attrs=TRUE}.
+#' Other arguments can be passed to that function. You may display a progress
+#' bar by setting \code{.progress=TRUE}.
 #'
 #' This object can be considered comparable to a 4-D \emph{NIfTI} file,
 #' particularly that returned by FSL's \emph{TBSS} \dQuote{prestats} step since
@@ -69,7 +70,7 @@ make_brainGraphList <- function(x, atlas, type=c('observed', 'random'),
 #'   \code{getOption('bg.progress')}
 #' @export
 #' @importFrom doParallel registerDoParallel
-#' @importFrom foreach getDoParRegistered
+#' @importFrom foreach getDoParRegistered getDoParWorkers
 #' @rdname brainGraphList
 
 make_brainGraphList.array <- function(x, atlas, type=c('observed', 'random'),
@@ -82,23 +83,31 @@ make_brainGraphList.array <- function(x, atlas, type=c('observed', 'random'),
   i <- NULL
   level <- match.arg(level)
   kNumGraphs <- dim(x)[3L]
-  if (is.null(gnames)) gnames <- seq_len(kNumGraphs)
-  stopifnot(length(gnames) == kNumGraphs)
-  gnames <- as.character(gnames)
+  if (is.null(gnames)) {
+    gnames <- pad_zeros(kNumGraphs)
+  } else {
+    stopifnot(length(gnames) == kNumGraphs)
+    gnames <- check_sID(gnames)
+  }
   if (is.null(grpNames)) {
-    grpNames <- if (level == 'group') gnames else rep(1, kNumGraphs)
+    grpNames <- if (level == 'group') gnames else rep.int('1', kNumGraphs)
+  } else {
+    stopifnot(length(grpNames) == kNumGraphs)
+    grpNames <- switch(class(grpNames),
+                       factor=as.character(grpNames),
+                       integer=, numeric=pad_zeros(grpNames),
+                       grpNames)
   }
-  stopifnot(length(grpNames) == kNumGraphs)
-  grpNames <- as.character(grpNames)
 
   type <- match.arg(type)
+  if (missing(atlas)) atlas <- guess_atlas(x)
   attrs <- c('atlas', 'type', 'level', 'modality', 'weighting', 'threshold')
   out <- setNames(vector('list', length(attrs)), attrs)
   for (a in attrs) {
     if (!is.null(get(a))) out[[a]] <- get(a)
   }
   out <- get_metadata(out)
-  if (!is.null(threshold)) threshold <- rep_len(threshold, kNumGraphs)
+  if (!is.null(threshold)) threshold <- rep.int(threshold, kNumGraphs)
 
   # Show a progress bar so you aren't left in the dark
   #---------------------------------------------------------
@@ -108,7 +117,7 @@ make_brainGraphList.array <- function(x, atlas, type=c('observed', 'random'),
   }
   loopfun <- make_brainGraph
   if (isTRUE(.progress)) {
-    ncpu <- getOption('bg.ncpus')
+    ncpu <- getDoParWorkers()
     print(paste('Start time:', format(as.POSIXct(Sys.time()), '%Y-%m-%d %H:%M:%OS0')))
     progbar <- txtProgressBar(min=0, max=kNumGraphs, style=3)
     loopfun <- function(...) {
@@ -121,7 +130,7 @@ make_brainGraphList.array <- function(x, atlas, type=c('observed', 'random'),
   }
 
   env <- environment()
-  counter <- 0
+  counter <- 0L
   g <- foreach(i=seq_len(kNumGraphs)) %dopar% {
     res <- loopfun(x[, , i], atlas, type, level, set.attrs, modality, weighting, threshold[i],
                    name=gnames[i], Group=grpNames[i], subnet=subnet, mode=mode,
@@ -146,7 +155,7 @@ make_brainGraphList.corr_mats <- function(x, atlas=x$atlas, type='observed',
                                           mode='undirected', weighted=NULL,
                                           diag=FALSE, .progress=getOption('bg.progress'), ...) {
   dims <- vapply(x$r.thresh, dim, numeric(3L))[3L, ]
-  if (any(dims > 1L)) x <- x[1]
+  if (any(dims > 1L)) x <- x[1L]
   A <- abind::abind(x$r.thresh)
   if (isTRUE(weighted)) A <- x$R * A
   out <- make_brainGraphList(A, atlas=atlas, type=type, level=level,
@@ -191,7 +200,7 @@ make_brainGraphList.bg_GLM <- function(x, atlas=x$atlas, type='observed',
                                        level='contrast', set.attrs=FALSE,
                                        modality=NULL, weighting=NULL,
                                        threshold=NULL, gnames=x$con.name, ...) {
-  contrast <- p <- p.fdr <- p.perm <- se <- stat <- region <- ESS <- NULL
+  Contrast <- contrast <- p <- p.fdr <- p.perm <- se <- stat <- region <- ESS <- NULL
   if (x$level == 'graph') stop('Not valid for graph-level results.')
 
   attrs <- c('atlas', 'type', 'level', 'modality', 'weighting', 'threshold')
@@ -201,25 +210,26 @@ make_brainGraphList.bg_GLM <- function(x, atlas=x$atlas, type='observed',
   }
   out <- get_metadata(out)
 
-  g.diffs <- vector('list', length=length(x$con.name))
+  g.diffs <- vector('list', length(gnames))
   for (i in seq_along(g.diffs)) {
-    g.diffs[[i]] <- make_empty_brainGraph(atlas, type='observed', level='contrast',
-                                          name=x$con.name[i], ...)
+    g.diffs[[i]] <- make_empty_brainGraph(atlas, type, level, modality, weighting,
+                                          threshold, name=gnames[i], ...)
     for (a in c('con.type', 'outcome', 'alt', 'alpha')) {
       g.diffs[[i]] <- set_graph_attr(g.diffs[[i]], a, x[[a]])
     }
 
-    V(g.diffs[[i]])$p <- 1 - x$DT[contrast == i, p]
-    V(g.diffs[[i]])$p.fdr <- 1 - x$DT[contrast == i, p.fdr]
+    DT <- x$DT[Contrast == gnames[i]]
+    V(g.diffs[[i]])$p <- 1 - DT$p
+    V(g.diffs[[i]])$p.fdr <- 1 - DT$p.fdr
     if (x$con.type == 't') {
-      V(g.diffs[[i]])$effect.size <- x$DT[contrast == i, gamma]
+      V(g.diffs[[i]])$effect.size <- DT$gamma
     } else {
-      V(g.diffs[[i]])$effect.size <- x$DT[contrast == i, ESS]
+      V(g.diffs[[i]])$effect.size <- DT$ESS
     }
-    V(g.diffs[[i]])$se <- x$DT[contrast == i, se]
-    V(g.diffs[[i]])$size2 <- x$DT[contrast == i, stat]
+    V(g.diffs[[i]])$se <- DT$se
+    V(g.diffs[[i]])$size2 <- DT$stat
     V(g.diffs[[i]])$size <- vec.transform(V(g.diffs[[i]])$size2, 0, 20)
-    if (isTRUE(x$permute)) V(g.diffs[[i]])$p.perm <- 1 - x$DT[contrast == i, p.perm]
+    if (isTRUE(x$permute)) V(g.diffs[[i]])$p.perm <- 1 - DT$p.perm
     class(g.diffs[[i]]) <- c('brainGraph_GLM', class(g.diffs[[i]]))
   }
   out$graphs <- g.diffs
@@ -233,7 +243,7 @@ make_brainGraphList.bg_GLM <- function(x, atlas=x$atlas, type='observed',
 #' @export
 #' @rdname glm_brainGraphList
 
-make_brainGraphList.mtpc <- function(x, atlas, type='observed', level='contrast',
+make_brainGraphList.mtpc <- function(x, atlas=x$atlas, type='observed', level='contrast',
                                      set.attrs=FALSE, modality=NULL, weighting=NULL,
                                      threshold=NULL, gnames=x$con.name, ...) {
   contrast <- region <- A.mtpc <- A.crit <- S.crit <- S.mtpc <- tau.mtpc <- NULL
@@ -246,10 +256,10 @@ make_brainGraphList.mtpc <- function(x, atlas, type='observed', level='contrast'
   }
   out <- get_metadata(out)
 
-  g.diffs <- vector('list', length=length(x$con.name))
+  g.diffs <- vector('list', length(gnames))
   for (i in seq_along(g.diffs)) {
     g.diffs[[i]] <- make_empty_brainGraph(atlas, type, level, modality, weighting,
-                                          threshold, name=gnames[i])
+                                          threshold, name=gnames[i], ...)
     for (a in c('con.type', 'outcome', 'alt', 'alpha')) {
       g.diffs[[i]] <- set_graph_attr(g.diffs[[i]], a, x[[a]])
     }
@@ -257,9 +267,9 @@ make_brainGraphList.mtpc <- function(x, atlas, type='observed', level='contrast'
       g.diffs[[i]] <- set_graph_attr(g.diffs[[i]], a, x$stats[contrast == i, get(a)])
     }
     V(g.diffs[[i]])$A.mtpc <- x$DT[contrast == i, unique(A.mtpc), by=region]$V1
-    V(g.diffs[[i]])$sig <- 0
+    V(g.diffs[[i]])$sig <- 0L
     sig.regions <- as.integer(x$DT[contrast == i & A.mtpc > A.crit, unique(region)])
-    V(g.diffs[[i]])[sig.regions]$sig <- 1
+    V(g.diffs[[i]])[sig.regions]$sig <- 1L
     class(g.diffs[[i]]) <- c('brainGraph_mtpc', class(g.diffs[[i]]))
   }
   out$graphs <- g.diffs
@@ -369,17 +379,18 @@ make_brainGraphList.NBS <- function(x, atlas, type='observed', level='contrast',
   warn_msg <- 'Indexing vector cannot have length greater than the number of groups.'
   # Subset groups; doesn't make sense for contrast lists
   if (!is.null(g) && x$level != 'contrast') {
-    if (is.factor(g)) g <- as.character(g)
     group.vec <- groups(x)
     grpNames <- unique(group.vec)
     kNumGroups <- length(grpNames)
     if (is.logical(g) && length(g) != kNumGroups) stop(stop_msg)
-    if (!is.logical(g) && length(g) > kNumGroups) {
+    if (length(g) > kNumGroups) {
       warning(warn_msg)
       g <- g[seq_len(kNumGroups)]
     }
-    if (is.numeric(g) || is.logical(g)) g <- grpNames[g]
-    if (is.character(g)) g <- which(group.vec %in% g)
+    g <- switch(class(g),
+                numeric=, integer=, logical=which(group.vec %in% grpNames[g]),
+                factor=which(group.vec %in% as.character(g)),
+                character=which(group.vec %in% g))
     x$graphs <- x$graphs[g]
   }
 
@@ -390,46 +401,63 @@ make_brainGraphList.NBS <- function(x, atlas, type='observed', level='contrast',
     i <- seq_len(kNumSubs)
   }
   if (is.logical(i) && length(i) != kNumSubs) stop(sub('group', 'subject', stop_msg))
-  if (!is.logical(i) && length(i) > kNumSubs) {
+  if (length(i) > kNumSubs) {
     warning(sub('group', 'subject', warn_msg))
     i <- i[seq_len(kNumSubs)]
   }
 
   if (is.character(i)) i <- which(names(x$graphs) %in% i)
-  x$graphs <- if (length(i) == 1 && isTRUE(drop)) x$graphs[[i]] else x$graphs[i]
+  x$graphs <- if (length(i) == 1L && isTRUE(drop)) x$graphs[[i]] else x$graphs[i]
   out <- if (isTRUE(drop)) x$graphs else x
   return(out)
 }
 
 #' @export
+#' @rdname brainGraphList
 print.brainGraphList <- function(x, ...) {
   kNumGraphs <- length(x$graphs)
   gnames <- names(x$graphs)
-  print_title_summary(paste0('A "brainGraphList" object of *', x$type,
-                             '* graphs containing ', kNumGraphs, ' ', x$level, 's.'))
+  print_title_summary('A "brainGraphList" object of *', x$type,
+                      '* graphs containing ', kNumGraphs, ' ', x$level, 's.')
 
   df <- print_bg_summary(x)
-  df <- df[-c(6, 10, 11, 13, 14), ]
+  df <- df[-c(6L, 10L, 11L, 13L, 14L), ]
   print(df, right=FALSE, row.names=FALSE)
   cat('\n')
 
   # Print subject/group names
   msg <- switch(x$level, subject='IDs:', 'names:')
   message(paste(simpleCap(x$level), msg))
-  if (kNumGraphs < 10) {
+  if (kNumGraphs < 10L) {
     print(gnames)
+    cat('\n')
   } else {
-    attrs.df <- print_text_vector(gnames, 6)
+    attrs.df <- print_text_vector(gnames, 6L)
     print(attrs.df, row.names=FALSE)
     cat('\n')
   }
-  if (x$level == 'subject' && 'Group' %in% graph_attr_names(x$graphs[[1]])) {
+  if (x$level == 'subject' && 'Group' %in% graph_attr_names(x$graphs[[1L]])) {
     message('Group membership:')
     print(table(groups(x)))
+    cat('\n')
+  }
+
+  if (x$type == 'random') {
+    message('# of random graphs:')
+    print(lengths(x$graphs))
   }
   invisible(x)
 }
 
+#' @export
+#' @rdname brainGraphList
+is.brainGraphList <- function(x) inherits(x, 'brainGraphList')
+
+#' @param object A \code{brainGraphList} object
+#' @export
+#' @rdname brainGraphList
+nobs.brainGraphList <- function(object, ...) length(object$graphs)
+
 #' Coerce list of graphs to a brainGraphList object
 #'
 #' \code{as_brainGraphList} coerces a list of graphs to a \code{brainGraphList}
@@ -461,25 +489,25 @@ as_brainGraphList <- function(g.list, type=c('observed', 'random'),
 
   type <- match.arg(type)
   level <- match.arg(level)
-  if (type == 'observed' || level == 'group') {
-    stopifnot(all(vapply(g.list, inherits, logical(1), 'brainGraph')))
-    g1 <- g.list[[1]]
-    ids <- vapply(g.list, graph_attr, character(1), 'name')
+  if (type == 'random') {
+    stopifnot(all(vapply(g.list, function(x) is_igraph(x[[1L]]), logical(1L))))
+    g1 <- g.list[[1L]][[1L]]
+    ids <- vapply(g.list, function(x) graph_attr(x[[1L]], 'name'), character(1L))
   } else {
-    stopifnot(all(vapply(g.list, function(x) is_igraph(x[[1]]), logical(1))))
-    g1 <- g.list[[1]][[1]]
-    ids <- vapply(g.list, function(x) graph_attr(x[[1]], 'name'), character(1))
+    stopifnot(all(vapply(g.list, is.brainGraph, logical(1L))))
+    g1 <- g.list[[1L]]
+    ids <- vapply(g.list, graph_attr, character(1L), 'name')
   }
 
   attrnames <- graph_attr_names(g1)
   attrs <- c('atlas', 'modality', 'weighting', 'threshold', 'version', 'sys', 'date')
-  out <- setNames(vector('list', length=9), c('type', 'level', attrs))
+  out <- setNames(vector('list', length=9L), c('type', 'level', attrs))
   out$type <- type
   out$level <- level
   for (x in attrs) {
     if (x %in% attrnames) out[[x]] <- graph_attr(g1, x)
   }
-  if (length(out$version < 3)) out$version <- list(r='', bg=out$version, ig='')
+  if (length(out$version) < 3L) out$version <- list(r='', bg=out$version, ig='')
   for (x in c('sys', 'date')) if (is.null(out[[x]])) out[[x]] <- ''
   out$graphs <- g.list
   names(out$graphs) <- ids
@@ -513,13 +541,13 @@ plot.brainGraphList <- function(x, plane, hemi, filename.base, diffs=FALSE, ...)
   pdf(file=sprintf('%s.pdf', filename.base), onefile=TRUE)
 
   kNumGraphs <- length(x$graphs)
-  plot(x[1], plane=plane, hemi=hemi, main=x[1]$name, ...)
-  for (i in seq.int(2, kNumGraphs)) {
+  plot(x[1L], plane=plane, hemi=hemi, main=x[1L]$name, ...)
+  for (i in seq.int(from=2L, to=kNumGraphs)) {
     plot(x[i], plane=plane, hemi=hemi, main=x[i]$name, ...)
     if (isTRUE(diffs)) {
-      g.diff <- graph.difference(x$graphs[[i]], x$graphs[[i-1]])
+      g.diff <- graph.difference(x$graphs[[i]], x$graphs[[i - 1L]])
       class(g.diff) <- c('brainGraph', class(g.diff))
-      ecols <- rep('deeppink', ecount(g.diff))
+      ecols <- rep.int('deeppink', ecount(g.diff))
       if (hasArg('edge.color')) ecols <- list(...)$edge.color
       plot(g.diff, add=TRUE, vertex.label=NA, vertex.shape='none',
            edge.width=5, edge.color=ecols, mni=FALSE, subtitle=NULL)
