@@ -27,22 +27,22 @@
 #'   variables
 #' @export
 #'
-#' @return A \code{\link[ggplot2]{ggplot}} object
+#' @return Either a \code{trellis} or \code{ggplot} object
 #' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
 
 plot_global <- function(g.list, xvar=c('density', 'threshold'), vline=NULL,
                         level.names='default', exclude=NULL, perms=NULL, alt='two.sided') {
-  sig <- trend <- yloc <- value <- variable <- threshold <- NULL
+  sig <- trend <- yloc <- value <- variable <- threshold <- panel.num <- NULL
 
   # Check if components are 'brainGraphList' objects
-  matches <- vapply(g.list, inherits, logical(1), 'brainGraphList')
+  matches <- vapply(g.list, is.brainGraphList, logical(1L))
   if (any(!matches)) stop("Input must be a list of 'brainGraphList' objects.")
 
   sID <- getOption('bg.subject_id')
   gID <- getOption('bg.group')
-  DT <- rbindlist(lapply(g.list, graph_attr_dt))
+  DT <- rbindlist(lapply(g.list, graph_attr_dt), use.names=TRUE)
   idvars <- c('atlas', 'modality', 'weighting', sID, gID, 'threshold', 'density')
-  idvars <- idvars[which(idvars %in% names(DT))]
+  idvars <- idvars[which(hasName(DT, idvars))]
   DT.m <- melt(DT, id.vars=idvars)
   DT.m <- droplevels(DT.m[!variable %in% exclude])
 
@@ -52,9 +52,9 @@ plot_global <- function(g.list, xvar=c('density', 'threshold'), vline=NULL,
     DT.m[, yloc := extendrange(value)[1L], by=variable]
     vars <- setdiff(names(perms$DT), 'densities')
     nvars <- length(vars)
-    if (length(alt) < nvars) alt <- rep(alt, length.out=nvars)
+    if (length(alt) < nvars) alt <- rep_len(alt, nvars)
     for (i in seq_along(vars)) {
-      dt <- plot(perms, measure=vars[i], alternative=alt[i])[[1]]$data[variable == 'obs.diff']
+      dt <- plot(perms, measure=vars[i], alternative=alt[i])[[1L]]$data[variable == 'obs.diff']
       DT.m[variable == vars[i], sig := dt$sig]
       DT.m[variable == vars[i], trend := dt$trend]
       DT.m[variable == vars[i], yloc := dt$yloc]
@@ -64,29 +64,74 @@ plot_global <- function(g.list, xvar=c('density', 'threshold'), vline=NULL,
   if (!is.null(level.names)) {
     if (level.names == 'default') level.names <- rename_levels(DT.m)
     nvars <- DT.m[, nlevels(variable)]
-    if (length(level.names) < nvars) level.names <- rep(level.names, length.out=nvars)
+    if (length(level.names) < nvars) level.names <- rep_len(level.names, nvars)
     levels(DT.m$variable) <- level.names
   }
 
   xvar <- match.arg(xvar)
-  p <- switch(xvar,
-              density=ggplot(DT.m, aes(x=density, y=value, col=get(gID))),
-              threshold=ggplot(DT.m, aes(x=threshold, y=value, col=get(gID))) + scale_x_reverse())
+  # 'base' plotting
+  if (!requireNamespace('ggplot2', quietly=TRUE)) {
+    grps <- DT.m[, levels(as.factor(get(gID)))]
+    DT.m[, panel.num := as.numeric(variable)]
+    if (!is.null(vline)) {
+      if (is.null(perms)) {
+        panelfun <- function(x, y, groups, ...) {
+          panel.abline(v=vline, lty=2, col='grey60')
+          panel.xyplot(x, y, groups, ...)
+        }
+      } else {
+        panelfun <- function(x, y, groups, ...) {
+          panel.num <- NULL
+          panel.abline(v=vline, lty=2, col='grey60')
+          panel.xyplot(x, y, groups, ...)
+          DT.m[panel.num == panel.number() & sig == '*',
+               panel.text(density, yloc, labels='*', col=plot.cols[1L])]
+          DT.m[panel.num == panel.number() & trend == '*',
+               panel.text(density, yloc, labels='*', col=plot.cols[2L])]
+        }
+      }
+    } else {
+      if (is.null(perms)) {
+        panelfun <- function(x, y, groups, ...) {
+          panel.xyplot(x, y, groups, ...)
+        }
+      } else {
+        panelfun <- function(x, y, groups, ...) {
+          panel.num <- NULL
+          panel.xyplot(x, y, groups, ...)
+          DT.m[panel.num == panel.number() & sig == '*',
+               panel.text(density, yloc, labels='*', col=plot.cols[1L])]
+          DT.m[panel.num == panel.number() & trend == '*',
+               panel.text(density, yloc, labels='*', col=plot.cols[2L])]
+        }
+      }
+    }
+    p <- xyplot(value ~ get(xvar) | variable, data=DT.m, groups=get(gID), type='l',
+                xlab=xvar, panel=panelfun, scales=list(y=list(relation='free')),
+                auto.key=list(space='bottom', title=gID, cex.title=1, columns=length(grps),
+                              lines=TRUE, points=FALSE))
 
-  if (sID %in% names(DT.m)) {
-    p <- p + stat_smooth(method='gam', formula=y~s(x))
+  # 'ggplot2' plotting
   } else {
-    p <- p + geom_line()
-  }
-  p <- p +
-    facet_wrap(~ variable, scales='free_y') +
-    theme(legend.position='bottom', axis.text.x=element_text(angle=45, hjust=1))
+    p <- switch(xvar,
+                density=ggplot2::ggplot(DT.m, ggplot2::aes(x=density, y=value, col=get(gID))),
+                threshold=ggplot2::ggplot(DT.m, ggplot2::aes(x=threshold, y=value, col=get(gID))) + ggplot2::scale_x_reverse())
 
-  if (!is.null(vline)) p <- p + geom_vline(xintercept=vline, lty=2, col='grey60')
-  if (!is.null(perms)) {
+    if (hasName(DT.m, sID)) {
+      p <- p + ggplot2::stat_smooth(method='gam', formula=y~s(x))
+    } else {
+      p <- p + ggplot2::geom_line()
+    }
     p <- p +
-      geom_text(aes(y=yloc, label=sig), col='red', size=3) +
-      geom_text(aes(y=yloc, label=trend), col='blue', size=3)
+      ggplot2::facet_wrap(~ variable, scales='free_y') +
+      ggplot2::theme(legend.position='bottom', axis.text.x=ggplot2::element_text(angle=45, hjust=1))
+
+    if (!is.null(vline)) p <- p + ggplot2::geom_vline(xintercept=vline, lty=2, col='grey60')
+    if (!is.null(perms)) {
+      p <- p +
+        ggplot2::geom_text(ggplot2::aes(y=yloc, label=sig), col='red', size=3) +
+        ggplot2::geom_text(ggplot2::aes(y=yloc, label=trend), col='blue', size=3)
+    }
   }
   return(p)
 }
