@@ -14,12 +14,11 @@
 #' @param show.points Logical indicating whether or not to show individual data
 #'   points (default: FALSE)
 #' @param ylabel A character string for the y-axis label
-#' @param ... Other arguments passed to \code{\link[ggplot2]{geom_boxplot}} or
-#'   \code{\link[ggplot2]{geom_violin}}
+#' @param ... Arguments passed to \code{geom_boxplot} or \code{geom_violin}
 #' @inheritParams plot_brainGraph_multi
 #' @export
 #'
-#' @return A \code{ggplot} object
+#' @return A \code{trellis} or \code{ggplot} object
 #'
 #' @author Christopher G. Watson, \email{cgwatson@@bu.edu}
 #' @examples
@@ -33,27 +32,38 @@ plot_vertex_measures <- function(g.list, measure, facet.by=NULL, group.by=getOpt
   variable <- value <- NULL
   gID <- getOption('bg.group')
 
-  if (!inherits(g.list, 'brainGraphList')) try(g.list <- as_brainGraphList(g.list))
+  if (!is.brainGraphList(g.list)) try(g.list <- as_brainGraphList(g.list))
   DT <- vertex_attr_dt(g.list)
-  stopifnot(measure %in% names(DT), group.by %in% names(DT))
+  stopifnot(all(hasName(DT, c(measure, group.by))))
   idvars <- c('atlas', 'modality', 'weighting', getOption('bg.subject_id'), gID, 'threshold',
               'density', 'region', 'lobe', 'hemi', 'class', 'network')
-  idvars <- idvars[which(idvars %in% names(DT))]
+  idvars <- idvars[which(hasName(DT, idvars))]
   DT.m <- melt(DT, id.vars=idvars)
   setnames(DT.m, group.by, 'group.by')
 
   type <- match.arg(type)
-  p <- ggplot(DT.m[variable == measure], aes(x=group.by, y=value, fill=group.by))
-  p <- if (type == 'violin') p + geom_violin(...) else p + geom_boxplot(...)
 
-  if (!is.null(facet.by)) {
-    stopifnot(facet.by %in% names(DT))
-    p <- p + facet_wrap(as.formula(paste('~', facet.by)), scales='free_y')
+  # 'base' plotting
+  if (!requireNamespace('ggplot2', quietly=TRUE)) {
+    panelfun <- if (type == 'violin') panel.violin else panel.bwplot
+    p <- bwplot(value ~ group.by | get(facet.by), data=DT.m[variable == measure],
+                panel=panelfun,
+                xlab=group.by, ylab=ylabel)
+
+  # 'ggplot2' plotting
+  } else {
+    p <- ggplot2::ggplot(DT.m[variable == measure], ggplot2::aes(x=group.by, y=value, fill=group.by))
+    p <- if (type == 'violin') p + ggplot2::geom_violin(...) else p + ggplot2::geom_boxplot(...)
+
+    if (!is.null(facet.by)) {
+      stopifnot(hasName(DT, facet.by))
+      p <- p + ggplot2::facet_wrap(as.formula(paste('~', facet.by)), scales='free_y')
+    }
+    if (isTRUE(show.points)) {
+      p <- p + ggplot2::geom_jitter(position=ggplot2::position_jitter(width=0.1, height=0))
+    }
+    p <- p + ggplot2::labs(x=group.by, y=ylabel) + ggplot2::theme(legend.position='none')
   }
-  if (isTRUE(show.points)) {
-    p <- p + geom_jitter(position=position_jitter(width=0.1, height=0))
-  }
-  p <- p + labs(x=group.by, y=ylabel) + theme(legend.position='none')
 
   return(p)
 }
